@@ -32,6 +32,12 @@
 
 namespace dasynq {
 
+namespace dprivate {
+class proc_status; // forward declaration
+}
+
+inline namespace v2 {
+
 template <class Base> class kqueue_loop;
 
 class kqueue_traits
@@ -68,7 +74,9 @@ class kqueue_traits
 #endif
 
         void set_signo(int signo) { info.si_signo = signo; }
-    };    
+    };
+
+    using proc_status_t = dasynq::dprivate::proc_status;
 
     class fd_r;
 
@@ -103,6 +111,8 @@ class kqueue_traits
     constexpr static bool supports_non_oneshot_fd = false;
 };
 
+} // namespace v2
+
 namespace dprivate {
 namespace dkqueue {
 
@@ -164,6 +174,8 @@ inline bool get_siginfo(int signo, siginfo_t *siginfo)
 } // namespace dkqueue
 } // namespace dprivate
 
+inline namespace v2 {
+
 template <class Base> class kqueue_loop : public Base
 {
     int kqfd = -1; // kqueue fd
@@ -615,6 +627,7 @@ template <class Base> class kqueue_loop : public Base
     }
 };
 
+} // namespace v2
 } // namespace dasynq
 
 #endif /* DASYNQ_KQUEUE_H_ */
