@@ -1,7 +1,7 @@
 #!/bin/sh
 # append parameter to file
 
-echo "$1" >> ./recorded-output
+echo "$1" >> "$IGR_OUTPUT"/recorded-output
 
 if [ "$1" = "part3" ] || [ "$1" = "part4" ]
 then
