@@ -108,6 +108,42 @@ public:
     }
 };
 
+#if SUPPORT_CAPABILITIES
+// A thin wrapper around the cap_iab_t structure to manage ownership (supports move)
+struct cap_iab_wrapper {
+    cap_iab_wrapper() {}
+    cap_iab_wrapper(std::string const &str) noexcept {
+        if (str.empty()) return;
+        // this may end up being nullptr
+        // throwing from constructors is bad, so always check .get() afterwards
+        iab = cap_iab_from_text(str.c_str());
+    }
+
+    cap_iab_wrapper(cap_iab_wrapper const &) = delete;
+    cap_iab_wrapper(cap_iab_wrapper &&v) noexcept: iab(v.iab) {
+        v.iab = nullptr;
+    }
+
+    cap_iab_wrapper &operator=(cap_iab_wrapper const &) = delete;
+    cap_iab_wrapper &operator=(cap_iab_wrapper &&v) noexcept {
+        iab = v.iab;
+        v.iab = nullptr;
+        return *this;
+    }
+
+    ~cap_iab_wrapper() noexcept {
+        if (iab) cap_free(iab);
+    }
+
+    cap_iab_t get() const noexcept {
+        return iab;
+    }
+
+private:
+    cap_iab_t iab = nullptr;
+};
+#endif
+
 // Complete read - read the specified size until end-of-file or error; continue read if
 // interrupted by signal.
 inline ssize_t complete_read(int fd, void * buf, size_t n)
