@@ -415,10 +415,17 @@ void do_system_shutdown(shutdown_type_t shutdown_type)
     int reboot_type = RB_AUTOBOOT; // reboot
 #if defined(RB_POWER_OFF)
     if (shutdown_type == shutdown_type_t::POWEROFF) reboot_type = RB_POWER_OFF;
+#elif defined(RB_POWEROFF)
+    // FreeBSD spells it slightly differently
+    if (shutdown_type == shutdown_type_t::POWEROFF) reboot_type = RB_POWEROFF;
+#elif defined(RB_POWERDOWN)
+    // NetBSD (at least) uses RB_POWERDOWN rather than RB_POWER_OFF
+    if (shutdown_type == shutdown_type_t::POWEROFF) reboot_type = RB_POWERDOWN;
 #endif
 #if defined(RB_HALT_SYSTEM)
     if (shutdown_type == shutdown_type_t::HALT) reboot_type = RB_HALT_SYSTEM;
 #elif defined(RB_HALT)
+    // NetBSD, FreeBSD
     if (shutdown_type == shutdown_type_t::HALT) reboot_type = RB_HALT;
 #endif
     
@@ -496,7 +503,11 @@ void do_system_shutdown(shutdown_type_t shutdown_type)
     
     sub_buf.append("Issuing shutdown via kernel...\n");
     loop.poll();  // give message a chance to get to console
+#ifdef __NetBSD__
+    reboot(reboot_type, NULL);
+#else
     reboot(reboot_type);
+#endif
 }
 
 // Watcher for subprocess output.
@@ -637,7 +648,11 @@ static loop_t::child_proc_watcher::proc_status_t run_process(const char * prog_a
 static void unmount_disks(loop_t &loop, subproc_buffer &sub_buf)
 {
     try {
+#ifdef __NetBSD__
+        const char * unmount_args[] = { "/sbin/umount", "-a", nullptr };
+#else
         const char * unmount_args[] = { "/bin/umount", "-a", "-r", nullptr };
+#endif
         run_process(unmount_args, loop, sub_buf);
     }
     catch (std::exception &e) {
@@ -650,7 +665,11 @@ static void unmount_disks(loop_t &loop, subproc_buffer &sub_buf)
 static void swap_off(loop_t &loop, subproc_buffer &sub_buf)
 {
     try {
+#ifdef __NetBSD__
+        const char * swapoff_args[] = { "/sbin/swapctl", "-U", nullptr };
+#else
         const char * swapoff_args[] = { "/sbin/swapoff", "-a", nullptr };
+#endif
         run_process(swapoff_args, loop, sub_buf);
     }
     catch (std::exception &e) {
