@@ -158,7 +158,7 @@ void test_proc_term_start()
 
     // One restart per 1000 interval
     p.set_restart_interval({1000, 0}, 1);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
 
     sset.add_service(&p);
 
@@ -228,7 +228,7 @@ void test_proc_term_restart()
 
     process_service p {&sset, "testproc", std::move(command), command_offsets, depends};
     init_service_defaults(p);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
     sset.add_service(&p);
 
     p.start();
@@ -283,7 +283,7 @@ void test_proc_term_restart2()
 
     process_service p {&sset, "testproc", std::move(command), command_offsets, depends};
     init_service_defaults(p);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
     sset.add_service(&p);
 
     b.add_dep(&p, WAITS);
@@ -369,7 +369,7 @@ void test_proc_term_restart3()
     sset.add_service(&p);
 
     service_record d1 {&sset, "test-service-2", service_type_t::INTERNAL, {{&p, REG}}};
-    d1.set_auto_restart(true);
+    d1.set_auto_restart(auto_restart_mode::ALWAYS);
     sset.add_service(&d1);
 
     d1.start();
@@ -412,7 +412,7 @@ void test_proc_term_restart4()
 
     process_service p {&sset, "testproc", std::move(command), command_offsets, depends};
     init_service_defaults(p);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
 
     time_val start_timeout {5, 0};
     p.set_start_timeout(start_timeout);
@@ -458,6 +458,109 @@ void test_proc_term_restart4()
     sset.remove_service(&p);
 }
 
+// Unexpected termination but only restarts on non-zero exit code.
+void test_proc_term_restart5()
+{
+    using namespace std;
+
+    service_set sset;
+
+    ha_string command = "test-command";
+    list<pair<unsigned,unsigned>> command_offsets;
+    command_offsets.emplace_back(0, command.length());
+    std::list<prelim_dep> depends;
+
+    process_service p {&sset, "testproc", std::move(command), command_offsets, depends};
+    init_service_defaults(p);
+
+    p.set_auto_restart(auto_restart_mode::ON_FAILURE);
+
+    sset.add_service(&p);
+
+    // Start the service
+    p.start();
+    sset.process_queues();
+    base_process_service_test::exec_succeeded(&p);
+    sset.process_queues();
+    assert(p.get_state() == service_state_t::STARTED);
+    assert(p.get_target_state() == service_state_t::STARTED);
+
+    // Unexpected termination - should restart because of 1 exit code
+    base_process_service_test::handle_exit(&p, 1);
+    sset.process_queues();
+    assert(p.get_target_state() == service_state_t::STARTED);
+    assert(p.get_state() == service_state_t::STARTING);
+    base_process_service_test::exec_succeeded(&p);
+    sset.process_queues();
+    assert(p.get_state() == service_state_t::STARTED);
+    assert(p.get_target_state() == service_state_t::STARTED);
+
+    // 2nd unexpected termination - should stop because of 0 exit code
+    base_process_service_test::handle_exit(&p, 0);
+    sset.process_queues();
+    assert(p.get_target_state() == service_state_t::STOPPED);
+    assert(p.get_state() == service_state_t::STOPPED);
+
+    sset.remove_service(&p);
+}
+
+// Unexpected termination but only restarts on specific signals.
+void test_proc_term_restart6()
+{
+    using namespace std;
+
+    service_set sset;
+
+    ha_string command = "test-command";
+    list<pair<unsigned,unsigned>> command_offsets;
+    command_offsets.emplace_back(0, command.length());
+    std::list<prelim_dep> depends;
+
+    process_service p {&sset, "testproc", std::move(command), command_offsets, depends};
+    init_service_defaults(p);
+
+    p.set_auto_restart(auto_restart_mode::ON_FAILURE);
+
+    sset.add_service(&p);
+
+    // Start the service
+    p.start();
+    sset.process_queues();
+    base_process_service_test::exec_succeeded(&p);
+    sset.process_queues();
+    assert(p.get_state() == service_state_t::STARTED);
+    assert(p.get_target_state() == service_state_t::STARTED);
+
+    // Unexpected termination - should restart because of SIGQUIT signal
+    base_process_service_test::handle_signal_exit(&p, SIGQUIT);
+    sset.process_queues();
+    assert(p.get_target_state() == service_state_t::STARTED);
+    assert(p.get_state() == service_state_t::STARTING);
+    base_process_service_test::exec_succeeded(&p);
+    sset.process_queues();
+    assert(p.get_state() == service_state_t::STARTED);
+    assert(p.get_target_state() == service_state_t::STARTED);
+
+    int signals[] = { SIGHUP, SIGINT, SIGUSR1, SIGUSR2, SIGTERM };
+    for (auto signal: signals) {
+        // unexpected termination - should stop because of specific signal
+        base_process_service_test::handle_signal_exit(&p, signal);
+        sset.process_queues();
+        assert(p.get_target_state() == service_state_t::STOPPED);
+        assert(p.get_state() == service_state_t::STOPPED);
+
+        // explicit restart
+        p.start();
+        sset.process_queues();
+        base_process_service_test::exec_succeeded(&p);
+        sset.process_queues();
+        assert(p.get_state() == service_state_t::STARTED);
+        assert(p.get_target_state() == service_state_t::STARTED);
+    }
+
+    sset.remove_service(&p);
+}
+
 // Failure to restart should propagate to dependent
 void test_proc_term_restart_fail()
 {
@@ -472,12 +575,12 @@ void test_proc_term_restart_fail()
 
     process_service p {&sset, "testproc", std::move(command), command_offsets, depends};
     init_service_defaults(p);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
 
     sset.add_service(&p);
 
     service_record p_dpt1 {&sset, "dpt1", service_type_t::INTERNAL, {{ &p, REG }}};
-    p_dpt1.set_auto_restart(true);
+    p_dpt1.set_auto_restart(auto_restart_mode::ALWAYS);
     sset.add_service(&p_dpt1);
 
     service_record p_dpt2 {&sset, "dpt2", service_type_t::INTERNAL, {{ &p_dpt1, WAITS }}};
@@ -1005,7 +1108,7 @@ void test_proc_smooth_recovery3()
     sset.add_service(&p);
 
     service_record d1 {&sset, "test-service-2", service_type_t::INTERNAL, {{&p, REG}}};
-    d1.set_auto_restart(true);
+    d1.set_auto_restart(auto_restart_mode::ALWAYS);
     sset.add_service(&d1);
 
     d1.start();
@@ -1417,6 +1520,73 @@ void test_bgproc_start_fail_pid()
     sset.remove_service(&p);
 }
 
+void test_bgproc_restart_x2()
+{
+    using namespace std;
+
+    service_set sset;
+
+    ha_string command = "test-command";
+    list<pair<unsigned,unsigned>> command_offsets;
+    command_offsets.emplace_back(0, command.length());
+    std::list<prelim_dep> depends;
+
+    bgproc_service p {&sset, "testproc", std::move(command), command_offsets, depends};
+    init_service_defaults(p);
+    p.set_pid_file("/run/daemon.pid");
+    sset.add_service(&p);
+
+    p.start();
+    sset.process_queues();
+
+    assert(p.get_state() == service_state_t::STARTING);
+
+    base_process_service_test::exec_succeeded(&p);
+    sset.process_queues();
+
+    assert(p.get_state() == service_state_t::STARTING);
+
+    supply_pid_contents("/run/daemon.pid");
+
+    base_process_service_test::handle_exit(&p, 0);
+
+    assert(p.get_state() == service_state_t::STARTED);
+    assert(event_loop.active_timers.size() == 0);
+
+    bp_sys::last_sig_sent = 0;
+
+    p.stop();
+    sset.process_queues();
+
+    assert(p.get_state() == service_state_t::STOPPING);
+    assert(bp_sys::last_sig_sent == SIGTERM);
+
+    base_process_service_test::handle_exit(&p, 0);
+    assert(p.get_state() == service_state_t::STOPPED);
+
+    // Start again:
+    p.start();
+    sset.process_queues();
+    assert(p.get_state() == service_state_t::STARTING);
+    base_process_service_test::exec_succeeded(&p);
+    sset.process_queues();
+    assert(p.get_state() == service_state_t::STARTING);
+    supply_pid_contents("/run/daemon.pid");
+    base_process_service_test::handle_exit(&p, 0);
+    assert(p.get_state() == service_state_t::STARTED);
+
+    // Stop again:
+    bp_sys::last_sig_sent = 0;
+    p.stop();
+    sset.process_queues();
+    assert(p.get_state() == service_state_t::STOPPING);
+    assert(bp_sys::last_sig_sent == SIGTERM);
+    base_process_service_test::handle_exit(&p, 0);
+    assert(p.get_state() == service_state_t::STOPPED);
+
+    sset.remove_service(&p);
+}
+
 void test_bgproc_unexpected_term()
 {
     using namespace std;
@@ -1793,7 +1963,7 @@ void test_bgproc_term_restart()
 
     bgproc_service p {&sset, "testproc", std::move(command), command_offsets, depends};
     init_service_defaults(p);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
     p.set_restart_delay(time_val {0, 1000});
     p.set_pid_file("/run/daemon.pid");
     sset.add_service(&p);
@@ -1856,7 +2026,7 @@ void test_bgproc_stop()
 
     bgproc_service p {&sset, "testproc", std::move(command), command_offsets, depends};
     init_service_defaults(p);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
     p.set_pid_file("/run/daemon.pid");
     sset.add_service(&p);
 
@@ -1905,7 +2075,7 @@ void test_bgproc_stop2()
 
     bgproc_service p {&sset, "testproc", std::move(command), command_offsets, depends};
     init_service_defaults(p);
-    p.set_auto_restart(true);
+    p.set_auto_restart(auto_restart_mode::ALWAYS);
     p.set_pid_file("/run/daemon.pid");
     sset.add_service(&p);
 
@@ -2492,6 +2662,8 @@ int main(int argc, char **argv)
     RUN_TEST(test_proc_term_restart2, "    ");
     RUN_TEST(test_proc_term_restart3, "    ");
     RUN_TEST(test_proc_term_restart4, "    ");
+    RUN_TEST(test_proc_term_restart5, "    ");
+    RUN_TEST(test_proc_term_restart6, "    ");
     RUN_TEST(test_proc_term_restart_fail, "");
     RUN_TEST(test_term_via_stop, "         ");
     RUN_TEST(test_term_via_stop2, "        ");
@@ -2511,6 +2683,7 @@ int main(int argc, char **argv)
     RUN_TEST(test_bgproc_start, "          ");
     RUN_TEST(test_bgproc_start_fail, "     ");
     RUN_TEST(test_bgproc_start_fail_pid, " ");
+    RUN_TEST(test_bgproc_restart_x2, "     ");
     RUN_TEST(test_bgproc_unexpected_term, "");
     RUN_TEST(test_bgproc_smooth_recover, " ");
     RUN_TEST(test_bgproc_smooth_recove2, " ");
